%% Genetic algorithm demo (Lecture 11)
% Hint: enable "cell mode" in menu: Cell -> Enable cell mode 
% Pressing <Ctrl><Enter> in a cell evaluates that cell.

%% Test function: Rastigin's function
% f(x,y) = 20 + x^2 + y^2 -10 * (cos(2*pi*x) + cos(2*pi*y))
fitfun = @(dum)rastriginsfcn(dum); % Take Rastrigin's function as fitness function.

% Evaluate and plot fitness function
x = linspace(-2, 2, 100); % Grid x
y = linspace(-2, 2, 100); % Grid y
z = zeros(length(y), length(x));
for ii = 1:length(x)
    for jj = 1:length(y)
        z(jj,ii) = fitfun([x(ii), y(jj)]);
    end
end

% Plot function (3D)
figure
subplot(121)
set(gca, 'fontsize', 15)
surf(x,y,z,'facecolor', 'interp'); daspect([1 1 30])
xlabel('x_1')
ylabel('x_2')
zlabel('z(x_1,x_2)')
title('Rastigin''s function (3D)')

% Plot function (Contour)
subplot(122)
set(gca, 'fontsize', 15)
contour(x,y,z); daspect([1 1 1])
title('Rastigin''s function (Contour)')
xlabel('x_1')
ylabel('x_2')
text(0, 0, '\leftarrow Global minimum', 'interpreter', 'tex', 'fontsize', 15)
text(0, 1, '\leftarrow Many local minima!', 'interpreter', 'tex', 'fontsize', 15)

set(gcf, 'units', 'normalized', 'position', [0.1 0.1 0.8 0.8])
drawnow;
%% Optimize using the genetic algorithm toolbox

options = gaoptimset;                   % Load default options
options = gaoptimset(options,...
    'populationsize', 20,...            % Number of individuals in population
    'elitecount', 2,...                 % Number of elites (elites move to the next generation without modification)
    'crossoverfraction', 0.8,...        % Fraction of population that is created by crossover [0..1]. Set too low: inbreeding; set too high: no convergence
    'popinitrange', [1; 2],...  % Initial range of the population (mutation can bring new generations out of this range)
    'plotfcn', @customplotfcn,...
    'plotinterval', 1);

figure(99); hold off
contour(x,y,z); daspect([1 1 1])
[x_est, fval, reason, output] = ga(fitfun, 2,[],[],[],[],[-2, -2],[2,2],[],options)